/*
 * Code: Ben Walding
 * Date: 1 January 1998
 *
 * This is database engine which should handle either CDDB or WorkMan database
 * I have no idea about CDDB, but WorkMan works.
 *
 * Protected by the GNU Public License v2.0  <distrib>/GPL.LICENSE 
 */


#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>

#include "workmandb.h"
#include "cdlabel.h"
#include "database.h"

Album *album_list;

void load_database(void);


/* Possibly better to implement this as classes with constructor / destructors etc */
/* Puts them in order, this makes it easier to print later on. */

Track::Track(Album *album) {
  Track *track_ptr = album->tracks;

  // Search to end but one of list.  If track_ptr is NULL from start, link new track to start.
  for (track_ptr = album->tracks; track_ptr; track_ptr = track_ptr->next) 
    if (!track_ptr->next) break;

  if (track_ptr == NULL) {
    album->tracks = this;
  } else {
    track_ptr->next = this;
  }

  next = NULL;
  titlestr = NULL;
  addr = 0;
  len = 0;

  return;
}

Track::~Track(void) {
  if (titlestr)
	free(titlestr);
}



Album::Album() {
  next = NULL;
  titlestr = NULL;
  artiststr = NULL;
  tracks = NULL;
  len = 0;
  
  next = album_list;
  album_list = this;
  return;
}

// Kill an Album, does not handle its own links, but doesn destroy child tracks
Album::~Album() {
  Track *trackPtr, *trackNext;

  if (titlestr)
	free(titlestr);

  titlestr = NULL; // These are hacks XXX XXX XXX

  if (artiststr)
	free(artiststr);
  artiststr = NULL; // These are hacks XXX XXX XXX

  for (trackPtr = tracks; trackPtr; trackPtr = trackNext) {
	trackNext = trackPtr->next;
	delete trackPtr;
  }
  
}

// Sorts a pre-existing set of albums
// Take the first in the old list, unlink it from old list, link it into new list
// Insertion sort => O(n^2)  => Still fast for small workman dbs and probably for anything less than 1000 entries
void sort_database(void) {
  Album *newlist = NULL;
  Album *albumPtr;
  Album *albumScan;

  if (!album_list)
	return;


  // Transfer the first record -> simplifies the rest to a single type.
  newlist = album_list;
  album_list = newlist->next;  
  newlist->next = NULL;


  while (album_list) {
	//Unlink head
	albumPtr = album_list;
	album_list = album_list->next;

	// Handle insertion into head of list
	if (albumPtr->compare(*newlist) < 0) {
	  albumPtr->next = newlist;
	  newlist = albumPtr;
	  continue;
	}

	// Handle insertion into middle / append
	for (albumScan = newlist; albumScan->next; albumScan = albumScan->next)
	  if (albumPtr->compare(*(albumScan->next)) < 0) 
		break;

	albumPtr->next = albumScan->next;
	albumScan->next = albumPtr;
  }
  album_list = newlist;
}

// Load the various databases
void load_database() {
  // Technically, is a clear required here? XXX XXX XXX
  clear_database();

  workman_load_database(global_options.workmandb);
  // cddb_load_database(global_options.cddb);

  /* Display error - XXX XXX XXX*/
  sort_database();
}


// Delete the database
void clear_database(void) {
  Album *albumPtr, *albumPtrNext;

  for (albumPtr = album_list; albumPtr; albumPtr = albumPtrNext) {
	albumPtrNext = albumPtr->next;
	delete albumPtr;
  }
  album_list = NULL;
  return;
}
  
Album *getAlbumIndex(int value) {
  Album *album_ptr;
  int index = 0;

  for (album_ptr = album_list; album_ptr && ++index <= value; album_ptr = album_ptr->next);

  return album_ptr;
}




